"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const types = require("./types-BA1Jqeol.cjs");
class DeferredTask {
  constructor() {
    this.promise = new Promise((resolve, reject) => {
      this.resolve = resolve;
      this.reject = reject;
    });
  }
  SetResult(value) {
    var _a;
    return (_a = this.resolve) == null ? void 0 : _a.call(void 0, value);
  }
  SetError(error) {
    var _a;
    return (_a = this.reject) == null ? void 0 : _a.call(void 0, error);
  }
  get Promise() {
    return this.promise;
  }
}
class RemoteError extends Error {
  constructor(error) {
    super(error.message);
    this.name = `${RemoteError.name}<${error.name}>`;
    this.stack = error.stack;
  }
}
class TimeoutError extends Error {
  constructor(message) {
    super(message);
    this.name = TimeoutError.name;
  }
}
async function CreateClient(url, timeout = 5e3) {
  const wsPolyFill = "ws";
  globalThis.WebSocket = globalThis.WebSocket ?? (await import(wsPolyFill)).WebSocket;
  return new class extends WebSocket {
    constructor() {
      super(url);
      this.callbacks = /* @__PURE__ */ new Map();
      this.rpc = {};
      this.rpc = new Proxy(this.rpc, {
        get: (_instance, property, _proxy) => {
          const method = property;
          return (...parameters) => {
            const reference = `${property}#${Date.now()}${Math.random()}`;
            const message = { method, reference, parameters };
            try {
              const task = new DeferredTask();
              this.callbacks.set(message.reference, task);
              setTimeout(() => task.SetError(new TimeoutError(`Failed to perform the RPC roundtrip for '${message.reference}' within the given timeout of ${timeout}ms!`)), timeout);
              this.send(JSON.stringify(message));
              return task.Promise;
            } catch (error) {
              this.callbacks.delete(message.reference);
              throw error;
            }
          };
        }
      });
      this.onmessage = async (event) => {
        try {
          const message = JSON.parse(event.data);
          const { type, reference, result, error } = message;
          if (type === types.MessageType.Callback && this.callbacks.has(reference)) {
            const callback = this.callbacks.get(reference);
            try {
              if (callback && error !== void 0) {
                return callback.SetError(new RemoteError(error));
              }
              if (callback && result !== void 0) {
                return callback.SetResult(result);
              }
            } finally {
              this.callbacks.delete(reference);
            }
          }
          throw new Error(`Failed to handle the received RPC message: '${JSON.stringify(message)}'`);
        } catch (error) {
          console.warn(error);
        }
      };
    }
    get RPC() {
      return this.rpc;
    }
  }();
}
exports.CreateClient = CreateClient;
